"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable:no-console
const fs = require("fs-extra");
const SwaggerParser = require("swagger-parser");
const parse_1 = require("./parse");
const render_1 = require("./render");
/**
 * Checks if a parsed spec is in OA2 format
 * @param specs a parsed spec
 *
 * @returns true or false
 */
function isOpenAPIV2(specs) {
    return specs.hasOwnProperty("swagger");
}
exports.isOpenAPIV2 = isOpenAPIV2;
/**
 * Wraps file writing to expose a common interface and log consistently
 * @param name name of the piece of code to render
 * @param outPath path of the file
 * @param code code to be saved
 *
 */
function writeGeneratedCodeFile(name, outPath, code) {
    console.log(`${name} -> ${outPath}`);
    return fs.writeFile(outPath, code);
}
/**
 * Module's main method. It generates files based on a given specification url
 * @param options
 *
 *
 */
function generateApi(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const { specFilePath, tsSpecFilePath, generateClient = false, definitionsDirPath, strictInterfaces = false, defaultSuccessType = "undefined", defaultErrorType = "undefined", camelCasedPropNames = false } = options;
        const { generateRequestTypes = generateClient, generateResponseDecoders = generateClient } = options;
        const api = yield SwaggerParser.bundle(specFilePath);
        if (!isOpenAPIV2(api)) {
            throw new Error("The specification is not of type swagger 2");
        }
        yield fs.ensureDir(definitionsDirPath);
        if (tsSpecFilePath) {
            yield writeGeneratedCodeFile("TS Spec", tsSpecFilePath, render_1.renderSpecCode(api));
        }
        const definitions = api.definitions;
        if (!definitions) {
            console.log("No definitions found, skipping generation of model code.");
            return;
        }
        yield Promise.all(Object.keys(definitions).map((definitionName) => render_1.renderDefinitionCode(definitionName, definitions[definitionName], strictInterfaces, camelCasedPropNames).then((code) => writeGeneratedCodeFile(definitionName, `${definitionsDirPath}/${definitionName}.ts`, code))));
        const needToParseOperations = generateClient || generateRequestTypes;
        if (needToParseOperations) {
            const specMeta = parse_1.parseSpecMeta(api);
            const allOperationInfos = parse_1.parseAllOperations(api, defaultSuccessType, defaultErrorType);
            if (generateRequestTypes) {
                yield writeGeneratedCodeFile("request types", `${definitionsDirPath}/requestTypes.ts`, render_1.renderAllOperations(allOperationInfos, generateResponseDecoders));
            }
            if (generateClient) {
                const code = yield render_1.renderClientCode(specMeta, allOperationInfos);
                yield writeGeneratedCodeFile("client", `${definitionsDirPath}/client.ts`, code);
            }
        }
    });
}
exports.generateApi = generateApi;
//# sourceMappingURL=index.js.map