/**
 * This module collects pure utility functions that convert a OpenAPI specification object into a shape which is convenient for code generation
 */
import { OpenAPIV2 } from "openapi-types";
import { IAuthHeaderParameterInfo, IHeaderParameterInfo, IOperationInfo, IParameterInfo, ISpecMetaInfo } from "./types";
/**
 * Extracts meta info in a convenient object
 * @param api
 */
export declare function parseSpecMeta(api: OpenAPIV2.Document): ISpecMetaInfo;
/**
 * Iterates over all operations in the specifications and returns a list of IOperationInfo struct describing them.
 * It also flattens global parameters and definitions by place them in each operation
 * @param api
 * @param defaultSuccessType
 * @param defaultErrorType
 *
 * @returns a list with all operation parsed
 */
export declare function parseAllOperations(api: OpenAPIV2.Document, defaultSuccessType: string, defaultErrorType: string): (IOperationInfo | undefined)[];
/**
 * Extracts all the info referring to a single operation and returns a IOperationInfo struct.
 * It may return undefined in case of unhandled specification or bad specification format
 *
 * @param api the whole spec
 * @param path the path of the current operation
 * @param extraParameters global parameters specified ad api-level
 * @param defaultSuccessType
 * @param defaultErrorType
 * @param operationKey identifies the operation for a path, correspond to a http method
 *
 * @returns a IOperationInfo struct if correct, undefined otherwise
 */
export declare const parseOperation: (api: OpenAPIV2.Document, path: string, extraParameters: (IParameterInfo | IHeaderParameterInfo)[], defaultSuccessType: string, defaultErrorType: string) => (operationKey: string) => IOperationInfo | undefined;
/**
 * Parse security along with security definitions to obtain a collection of tuples in the form (keyName, headerName).
 * It works with security object both global and operation-specific.
 * see: https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#securityRequirementObject
 * https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#securityDefinitionsObject
 * @param securityDefinitions global security definition objects
 * @param security global or specific security requirements
 *
 * @returns Array of tuples in the form (keyName, headerName). Example: [{ e1: 'token', e2: 'Authorization'}]
 */
export declare function getAuthHeaders(securityDefinitions: OpenAPIV2.Document["securityDefinitions"], security?: OpenAPIV2.SecurityRequirementObject[]): IAuthHeaderParameterInfo[];
