"use strict";
/**
 * This module collects pure utility functions that convert a OpenAPI specification object into a shape which is convenient for code generation
 */
Object.defineProperty(exports, "__esModule", { value: true });
const tuples_1 = require("italia-ts-commons/lib/tuples");
const utils_1 = require("../../lib/utils");
/**
 * Extracts meta info in a convenient object
 * @param api
 */
function parseSpecMeta(api) {
    var _a, _b;
    return {
        basePath: api.basePath,
        version: (_a = api.info) === null || _a === void 0 ? void 0 : _a.version,
        title: (_b = api.info) === null || _b === void 0 ? void 0 : _b.title
    };
}
exports.parseSpecMeta = parseSpecMeta;
/**
 * Iterates over all operations in the specifications and returns a list of IOperationInfo struct describing them.
 * It also flattens global parameters and definitions by place them in each operation
 * @param api
 * @param defaultSuccessType
 * @param defaultErrorType
 *
 * @returns a list with all operation parsed
 */
function parseAllOperations(api, defaultSuccessType, defaultErrorType) {
    // map global auth headers only if global security is defined
    const globalAuthHeaders = api.security
        ? getAuthHeaders(api.securityDefinitions, api.security)
        : [];
    return Object.keys(api.paths)
        .map(path => {
        const pathSpec = api.paths[path];
        const extraParameters = [
            ...parseExtraParameters(pathSpec),
            ...globalAuthHeaders
        ];
        return Object.keys(pathSpec)
            .map(exports.parseOperation(api, path, extraParameters, defaultSuccessType, defaultErrorType))
            .filter(Boolean);
    })
        .reduce((flatten, elems) => flatten.concat(elems), []);
}
exports.parseAllOperations = parseAllOperations;
/**
 * It extracts global parameters from a path definition. Parameters in body, path, query and form are of type IParameterInfo, while header parameters are of type IHeaderParameterInfo
 * @param pathSpec a pat definition
 *
 * @returns a list of parameters that applies to all methods of a path. Header parameters ship a more complete structure.
 */
const parseExtraParameters = (pathSpec) => {
    return typeof pathSpec.parameters !== "undefined"
        ? pathSpec.parameters.reduce((prev, param) => {
            if (param && param.type) {
                const paramName = `${param.name}${param.required === true ? "" : "?"}`;
                return [
                    ...prev,
                    {
                        headerName: param.in === "header" ? paramName : undefined,
                        in: param.in,
                        name: paramName,
                        type: specTypeToTs(param.type)
                    }
                ];
            }
            return prev;
        }, [])
        : [];
};
/**
 * Extracts all the info referring to a single operation and returns a IOperationInfo struct.
 * It may return undefined in case of unhandled specification or bad specification format
 *
 * @param api the whole spec
 * @param path the path of the current operation
 * @param extraParameters global parameters specified ad api-level
 * @param defaultSuccessType
 * @param defaultErrorType
 * @param operationKey identifies the operation for a path, correspond to a http method
 *
 * @returns a IOperationInfo struct if correct, undefined otherwise
 */
exports.parseOperation = (api, path, extraParameters, defaultSuccessType, defaultErrorType) => (operationKey) => {
    const { parameters: specParameters, securityDefinitions } = api;
    const pathSpec = api.paths[path];
    const method = operationKey.toLowerCase();
    const operation = method === "get"
        ? pathSpec.get
        : method === "post"
            ? pathSpec.post
            : method === "put"
                ? pathSpec.put
                : method === "delete"
                    ? pathSpec.delete
                    : undefined;
    if (operation === undefined) {
        console.warn(`Skipping unsupported method [${method}]`);
        return;
    }
    const operationId = operation.operationId;
    if (typeof operationId === "undefined") {
        console.warn(`Skipping method with missing operationId [${method}]`);
        return;
    }
    const importedTypes = getImportedTypes(operation.parameters);
    const operationParams = typeof operation.parameters !== "undefined"
        ? operation.parameters
            .map(parseParameter(specParameters, operationId))
            .filter((e) => typeof e !== "undefined")
        : [];
    const authHeadersAndParams = operation.security
        ? getAuthHeaders(securityDefinitions, operation.security)
        : [];
    const authParams = authHeadersAndParams;
    const parameters = [...extraParameters, ...authParams, ...operationParams];
    const contentTypeHeaders = (method === "post" || method === "put") &&
        Object.keys(operationParams).length > 0
        ? ["Content-Type"]
        : [];
    const authHeaders = authHeadersAndParams.map(pick("headerName"));
    const extraHeaders = extraParameters
        .filter((p) => p.in === "header")
        .map(pick("headerName"));
    const headers = [...contentTypeHeaders, ...authHeaders, ...extraHeaders];
    const responses = Object.keys(operation.responses).map(responseStatus => {
        const response = operation.responses[responseStatus];
        const typeRef = response.schema ? response.schema.$ref : undefined;
        const responseHeaders = Object.keys(response.headers || {});
        const parsedRef = typeRef ? typeFromRef(typeRef) : undefined;
        if (parsedRef !== undefined) {
            importedTypes.add(parsedRef.e2);
        }
        const responseType = parsedRef
            ? parsedRef.e2
            : responseStatus === "200"
                ? defaultSuccessType
                : defaultErrorType;
        return tuples_1.Tuple3(responseStatus, responseType, responseHeaders);
    });
    const consumes = method === "get"
        ? undefined // get doesn't need content type as it does not ships a body
        : operation.consumes && operation.consumes.length
            ? operation.consumes[0]
            : api.consumes && api.consumes.length
                ? api.consumes[0]
                : "application/json"; // use json as default for methods that requires a Content-Type header
    const produces = operation.produces && operation.produces.length
        ? operation.produces[0]
        : api.produces && api.produces.length
            ? api.produces[0]
            : undefined;
    return {
        method,
        operationId,
        headers,
        parameters,
        responses,
        importedTypes,
        path,
        consumes,
        produces
    };
};
/**
 * Parse a request parameter into an IParameterInfo structure.
 * The function has the curried form (specParameters, operationId) -> (param) -> IParameterInfo
 * @param specParameters spec's global parameters
 * @param operationId the identifier for the operation. Used for logging purpose
 * @param param the request parameter to parse
 *
 * @returns a struct describing the parameter
 *
 * @example
 * // The param is defined inline in the operation
 * ({}, 'myOperationId') ->
 *   ({ name: 'qo', in: 'query', required: false, type: 'string' }) ->
 *    { name: 'qo?', in: 'query', type: 'string' }
 * @example
 * // The param is defined as a reference to a global definition
 * ({}, 'myOperationId') ->
 *   ({
 *     in: 'body',
 *     name: 'body',
 *     schema: { '$ref': '#/definitions/MySchema' },
 *     required: true
 *   }) ->
 *    { name: 'MySchema', type: 'MySchema', in: 'body' }
 * @example
 * // The param is a reference to a global parameter
 * (
 *  { PaginationRequest: { name: 'cursor', in: 'query', type: 'string' } },
 *  'myOperationId'
 * ) ->
 *   ({ '$ref': '#/parameters/PaginationRequest' }) ->
 *    { name: 'cursor?', type: 'string', in: 'query' }
 */
const parseParameter = (specParameters, operationId) => (param) => param.name && param.type
    ? parseInlineParam(param)
    : parseParamWithReference(specParameters, operationId, param);
const parseInlineParam = (param) => (Object.assign({ in: param.in, name: `${param.name}${param.required ? "" : "?"}`, type: specTypeToTs(param.type) }, (param.in === "header" ? { headerName: param.name } : {})));
const parseParamWithReference = (specParameters, operationId, param) => {
    const refInParam = param.$ref || (param.schema ? param.schema.$ref : undefined);
    if (refInParam === undefined) {
        console.warn(`Skipping param without ref in operation [${operationId}] [${param.name}]`);
        return undefined;
    }
    const parsedRef = typeFromRef(refInParam);
    if (parsedRef === undefined) {
        console.warn(`Cannot extract type from ref [${refInParam}]`);
        return undefined;
    }
    const refType = parsedRef.e1;
    if (refType === "other") {
        console.warn(`Unrecognized ref type [${refInParam}]`);
        return undefined;
    }
    const paramType = refType === "definition"
        ? parsedRef.e2
        : specParameters
            ? specTypeToTs(specParameters[parsedRef.e2].type)
            : undefined;
    if (paramType === undefined) {
        console.warn(`Cannot resolve parameter ${parsedRef.e2}`);
        return undefined;
    }
    const isParamRequired = refType === "definition"
        ? param.required === true
        : specParameters
            ? specParameters[parsedRef.e2].required
            : false;
    const paramName = utils_1.uncapitalize(specParameters && specParameters[parsedRef.e2]
        ? specParameters[parsedRef.e2].name
        : param.name);
    const paramIn = specParameters && specParameters[parsedRef.e2]
        ? specParameters[parsedRef.e2].in
        : param.in;
    return Object.assign({ in: paramIn, name: `${paramName}${isParamRequired ? "" : "?"}`, type: paramType }, (paramIn === "header" ? { headerName: paramName } : {}));
};
/**
 * Parse security along with security definitions to obtain a collection of tuples in the form (keyName, headerName).
 * It works with security object both global and operation-specific.
 * see: https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#securityRequirementObject
 * https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#securityDefinitionsObject
 * @param securityDefinitions global security definition objects
 * @param security global or specific security requirements
 *
 * @returns Array of tuples in the form (keyName, headerName). Example: [{ e1: 'token', e2: 'Authorization'}]
 */
function getAuthHeaders(securityDefinitions, security) {
    const securityKeys = security && security.length
        ? security
            .map((_) => Object.keys(_)[0])
            .filter(_ => _ !== undefined)
        : undefined;
    const securityDefs = securityKeys !== undefined && securityDefinitions !== undefined
        ? // If we have both security and securityDefinitions defined, we extract
            // security items mapped to their securityDefinitions definitions.
            securityKeys.map(k => tuples_1.Tuple2(k, securityDefinitions[k]))
        : securityDefinitions !== undefined
            ? Object.keys(securityDefinitions).map(k => tuples_1.Tuple2(k, securityDefinitions[k]))
            : [];
    return securityDefs
        .filter(_ => _.e2 !== undefined)
        .filter(_ => _.e2.in === "header")
        .map(_ => {
        const { name: headerName, type: tokenType, ["x-auth-scheme"]: authScheme = "none" } = _.e2; // Because _.e2 is of type OpenAPIV2.SecuritySchemeObject which is the super type of OpenAPIV2.SecuritySchemeApiKey. In the previous step of the chain we filtered so we're pretty sure _.e2 is of type OpenAPIV2.SecuritySchemeApiKey, but the compiler fails at it. I can add an explicit guard to the filter above, but I think the result is the same.
        return {
            authScheme,
            headerName,
            // tslint:disable-next-line: no-useless-cast
            in: "header",
            name: _.e1,
            tokenType,
            type: "string"
        };
    });
}
exports.getAuthHeaders = getAuthHeaders;
/**
 * Takes an array of parameters and collect each definition referenced.
 * Those will correspond to types to be imported in typescript
 * @param parameters
 *
 * @returns a set of definitions to be imported
 *
 * @example
 * ([
 *  {"in":"body","name":"body","schema":{"$ref":"#/definitions/MySchema"},"required":true},
 *  {"$ref":"#/parameters/MyParam"}
 * ]) -> Set { 'MySchema' }
 */
const getImportedTypes = (parameters) => new Set(typeof parameters !== "undefined"
    ? parameters
        .map(paramParsedRef)
        .reduce((prev, parsed) => {
        if (typeof parsed === "undefined") {
            return prev;
        }
        const { e1: refType, e2 } = parsed;
        return refType === "definition" && typeof e2 !== "undefined"
            ? prev.concat(e2)
            : prev;
    }, [])
    : []);
/**
 * Given a request param, parses its schema reference, if any
 * @param param a request parameter
 *
 * @returns an ITuple<refType, refName> if the parameter has a reference, undefined otherwise
 */
const paramParsedRef = (param) => {
    if (typeof param === "undefined") {
        return undefined;
    }
    const refInParam = param.$ref || (param.schema ? param.schema.$ref : undefined);
    if (typeof refInParam === "undefined") {
        return undefined;
    }
    return typeFromRef(refInParam);
};
/**
 * Given a string in the form "#/<refType>/<refName>/, it returns a tuple in the form (refType, refName)"
 * @param s
 *
 * @returns an ITuple object with { e1: refType, e2: refName }, undefined if the string is not the the correct form
 */
function typeFromRef(s) {
    const parts = s.split("/");
    if (parts && parts.length === 3) {
        const refType = parts[1] === "definitions"
            ? "definition"
            : parts[1] === "parameters"
                ? "parameter"
                : "other";
        return tuples_1.Tuple2(refType, parts[2]);
    }
    return undefined;
}
/**
 * Given an OpenAPI param type, it returns its Typescript correspondent
 * @param t
 *
 * @returns a Typescript type
 */
function specTypeToTs(t) {
    switch (t) {
        case "integer":
            return "number";
        case "file":
            return `{ "uri": string, "name": string, "type": string }`;
        default:
            return t;
    }
}
/**
 * Pick a field from an object
 * @param field field to pick
 * @param elem the base object
 */
const pick = (field) => (elem) => elem[field];
//# sourceMappingURL=parse.js.map