/**
 * This module collects pure utility functions that render a code text based of a data structure.
 * Each function return a string containing formatted code, but does not write the code file.
 * Code generation might use template string literals or nunjucks templates.
 * Data structure might be a parsed structure or directly a OpenAPI object (in which case, the parsing logic is demanded to the template macros).
 */
import { OpenAPIV2 } from "openapi-types";
import { IOperationInfo, ISpecMetaInfo } from "./types";
/**
 * Render a code block which exports an object literal representing the api specification
 *
 * @param spec the original api specification
 */
export declare function renderSpecCode(spec: OpenAPIV2.Document): string;
/**
 * Given a OpenAPI definition object, it renders the code which describes the correspondent typescript model
 * @param definitionName the name of the definition
 * @param definition the definition data
 * @param strictInterfaces wheater requires strict interfaces or not
 * @param camelCasedPropNames wheater model properties must me camel-cased.
 *
 * @returns the formatted code for the model's typescript definition
 */
export declare function renderDefinitionCode(definitionName: string, definition: OpenAPIV2.DefinitionsObject, strictInterfaces: boolean, camelCasedPropNames?: boolean): Promise<string>;
/**
 * Given a list of parsed operations, it renders the code for an opinionated http client module that imlements each operation as an async method
 * @param specMeta meta info of the api specification
 * @param operations the list of parsed operations
 *
 * @returns the code of a http client
 */
export declare function renderClientCode(specMeta: ISpecMetaInfo, operations: Array<IOperationInfo | undefined>): Promise<string>;
/**
 * Renders the code that includes every operation definition
 * @param allOperationInfos collection of parsed operations
 * @param generateResponseDecoders true to include decoders
 *
 * @return the rendered code
 */
export declare function renderAllOperations(allOperationInfos: Array<IOperationInfo | undefined>, generateResponseDecoders: boolean): string;
/**
 * Render the code of decoders and request types of a single operation
 * @param operationInfo
 * @param generateResponseDecoders true if decoders have to be added
 *
 * @returns a tuple containing the generated code and the
 */
export declare const renderOperation: (operationInfo: IOperationInfo, generateResponseDecoders: boolean) => string;
/**
 * Compose the code for response decoder of an operation
 * @param operationInfo the operation
 *
 * @returns {string} the composed code
 */
export declare function renderDecoderCode({ responses, operationId }: IOperationInfo): string;
