/**
 * This module is a collection of custom filters to be used in Nunjucks templates
 */
/**
 * Wheater or not an array contains an item
 * @param subject the provided array
 * @param item item to search
 *
 * @return true if item is found
 */
export declare const contains: <T>(subject: readonly T[], item: T) => boolean;
/**
 * Wheater or not the first caracter of the string is the provided item
 * @param subject
 * @param item
 *
 * @returns true or false
 */
export declare const startsWith: (subject: string, item: string) => boolean;
/**
 * First letter to uppercase
 *
 * @param subject string to capitalize
 */
export declare const capitalizeFirst: (subject: string) => string;
/**
 * Wraps given text in a block comment
 * @param subject the given text
 *
 * @returns the wrapped comment
 */
export declare const comment: (subject: string) => string;
/**
 * Converts a string to camel cased
 * @param subject provided string
 *
 * @returns camel cased string
 *
 */
export declare const camelCase: (subject: string) => string;
/**
 * Sanitise a string to be used as a javascript identifier
 * see https://developer.mozilla.org/en-US/docs/Glossary/Identifier#:~:text=An%20identifier%20is%20a%20sequence,not%20start%20with%20a%20digit.
 *
 * @param subject provided string or array of strings
 *
 * @returns Sanitised string or array of sanitised strings
 *
 * @example
 * ("9-my invalid_id1") -> "myInvalidId1"
 */
export declare const safeIdentifier: (subject: string | readonly string[]) => string | string[] | undefined;
/**
 * Sanitise a string to be used as an object field name when destructuring.
 * The use case is when the template is composing a function declaration and the parameter is destructured
 * @param subject provided string or array of strings
 *
 * @returns Sanitised string or array of sanitised strings
 *
 * @example
 * ("9-my invalid_id1") -> "[\"9-my invalid_id1\"]: myInvalidId1"
 */
export declare const safeDestruct: (subject: string | readonly string[]) => string | string[] | undefined;
/**
 * Object.keys
 * @param subject provided object
 *
 * @returns a list of keys
 */
export declare const keys: (subject: object) => string[];
/**
 * The first element of an array, if defined
 * @param subject
 *
 * @return the first element if present, undefined otherwise
 */
export declare const first: <T>(subject: readonly T[] | undefined) => T | undefined;
/**
 * Array.join
 * @param subject
 * @param separator
 *
 * @return the joined string
 */
export declare const join: <T>(subject: readonly T[], separator: string) => string;
/**
 * Given an array, returns all the elements except the first
 * example: [1,2,3] -> [2,3]
 * example: [] -> []
 * @param subject provided array
 *
 * @returns the array tail
 */
export declare const tail: <T>(subject: readonly T[]) => T[];
/**
 * Returns an array containing all the elements of a given array plus the element to push
 * @param subject provided array
 * @param toPush element to push
 */
export declare const push: <T, R>(subject: readonly T[] | undefined, toPush: T | R) => (T | R)[];
/**
 * Given an hash set, returns the value of a given key. If an array of hash sets is given, an array of values os returned
 * @param subject a hash set or an array of hash set
 * @param key the key to pick
 *
 * @returns a value or an array of value
 */
export declare const pick: <T extends Record<string, any>>(subject: T | T[] | undefined, key: keyof T) => T[keyof T] | T[keyof T][];
