"use strict";
/**
 * This module is a collection of custom filters to be used in Nunjucks templates
 */
Object.defineProperty(exports, "__esModule", { value: true });
const safe_identifier_1 = require("safe-identifier");
const utils_1 = require("../utils");
/**
 * Apply a filter function indiscriminately to a single subject or to an array of subjects
 * In most cases nunjucks filters work for both strings or array of strings, so it's worth to handle this mechanism once forever
 * @param subject
 */
const oneOrMany = (filterFn) => (subject) => !subject
    ? undefined
    : typeof subject === "string"
        ? filterFn(subject)
        : subject.map(filterFn);
/**
 * Wheater or not an array contains an item
 * @param subject the provided array
 * @param item item to search
 *
 * @return true if item is found
 */
exports.contains = (subject, item) => subject.indexOf(item) !== -1;
/**
 * Wheater or not the first caracter of the string is the provided item
 * @param subject
 * @param item
 *
 * @returns true or false
 */
exports.startsWith = (subject, item) => subject.indexOf(item) === 0;
/**
 * First letter to uppercase
 *
 * @param subject string to capitalize
 */
exports.capitalizeFirst = (subject) => `${subject[0].toUpperCase()}${subject.slice(1)}`;
/**
 * Wraps given text in a block comment
 * @param subject the given text
 *
 * @returns the wrapped comment
 */
exports.comment = (subject) => `/**\n * ${subject.split("\n").join("\n * ")} \n */`;
/**
 * Converts a string to camel cased
 * @param subject provided string
 *
 * @returns camel cased string
 *
 */
exports.camelCase = (subject) => subject.replace(/(\_\w)/g, ([, firstLetter]) => typeof firstLetter === "undefined" ? "" : firstLetter.toUpperCase());
/**
 * Sanitise a string to be used as a javascript identifier
 * see https://developer.mozilla.org/en-US/docs/Glossary/Identifier#:~:text=An%20identifier%20is%20a%20sequence,not%20start%20with%20a%20digit.
 *
 * @param subject provided string or array of strings
 *
 * @returns Sanitised string or array of sanitised strings
 *
 * @example
 * ("9-my invalid_id1") -> "myInvalidId1"
 */
exports.safeIdentifier = oneOrMany(subject => utils_1.pipe((v) => v.replace(/^[0-9]+/, ""), safe_identifier_1.identifier, exports.camelCase)(subject));
/**
 * Sanitise a string to be used as an object field name when destructuring.
 * The use case is when the template is composing a function declaration and the parameter is destructured
 * @param subject provided string or array of strings
 *
 * @returns Sanitised string or array of sanitised strings
 *
 * @example
 * ("9-my invalid_id1") -> "[\"9-my invalid_id1\"]: myInvalidId1"
 */
exports.safeDestruct = oneOrMany((subject) => `["${subject}"]: ${exports.safeIdentifier(subject)}`);
/**
 * Object.keys
 * @param subject provided object
 *
 * @returns a list of keys
 */
exports.keys = (subject) => {
    return subject ? Object.keys(subject) : [];
};
/**
 * The first element of an array, if defined
 * @param subject
 *
 * @return the first element if present, undefined otherwise
 */
exports.first = (subject) => {
    return subject ? subject[0] : undefined;
};
/**
 * Array.join
 * @param subject
 * @param separator
 *
 * @return the joined string
 */
exports.join = (subject, separator) => {
    return subject.join(separator);
};
/**
 * Given an array, returns all the elements except the first
 * example: [1,2,3] -> [2,3]
 * example: [] -> []
 * @param subject provided array
 *
 * @returns the array tail
 */
exports.tail = (subject) => subject && subject.length ? subject.slice(1) : [];
/**
 * Returns an array containing all the elements of a given array plus the element to push
 * @param subject provided array
 * @param toPush element to push
 */
exports.push = (subject, toPush) => (subject && subject.length ? [...subject, toPush] : [toPush]);
/**
 * Given an hash set, returns the value of a given key. If an array of hash sets is given, an array of values os returned
 * @param subject a hash set or an array of hash set
 * @param key the key to pick
 *
 * @returns a value or an array of value
 */
exports.pick = (subject, key) => !subject
    ? []
    : Array.isArray(subject)
        ? subject.map(({ [key]: value }) => value)
        : subject[key];
//# sourceMappingURL=filters.js.map